var html = "<section class=\"mt-10\"><h1 id=\"testing-solid\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#testing-solid\">#</a> Testing Solid</h1><p>To use your Solid code in production, it needs to be tested. Since you don't want to test everything manually, you need automated tests. This guide describes how to set everything up and a few useful patterns for testing Solid code.</p><h2 id=\"testing-setup\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#testing-setup\">#</a> Testing Setup</h2><p>We offer support for two test runners:</p><ul> <li> <p>jest - very well established with many features</p> </li> <li> <p>uvu - only brings the bare necessities</p> </li> </ul><p>Both are based around <a href=\"https://github.com/solidjs/solid-testing-library\">solid-testing-library</a>, which integrates <a href=\"https://testing-library.com/\">Testing Library</a> into Solid. Testing Library mimics a light-weight browser and provides an API to interact with it from your tests.</p><p>We maintain a starter template for Solid and Jest tests. We recommend you base your project on it, or alternatively install the starter template in a scratch project and copy the configuration from it into your own project.</p><p>The templates use the <a href=\"https://github.com/Rich-Harris/degit\">degit</a> utility for installation.</p><h3 id=\"setting-up-jest\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#setting-up-jest\">#</a> Setting up Jest</h3><p>The Jest integration is based around the <a href=\"https://github.com/solidjs/solid-jest\">solid-jest/preset/browser</a> Jest configuration preset which lets you use Solid as in the browser. This uses babel to transform the Solid code.</p><p>It uses <a href=\"https://github.com/testing-library/jest-dom\">jest-dom</a> to extend <code>expect</code> with a bunch of custom matchers that help you write tests.</p><h4 id=\"jest-with-typescript-(ts-jest)\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#jest-with-typescript-(ts-jest)\">#</a> Jest with TypeScript (<code>ts-jest</code>)</h4><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#24292e\">$ npx degit solidjs/templates/ts-jest my-solid-project</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">$ </span><span style=\"color:#005cc5\">cd</span><span style=\"color:#24292e\"> my-solid-project</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">$ npm install </span><span style=\"color:#6a737d\"># or pnpm install or yarn install</span></span>\n<span class=\"line\"></span></code></pre><p>Note that this template does not do typechecks during testing; you can use your IDE or a custom <code>tsc --noEmit</code> script in <code>package.json</code> to trigger such checks.</p><h3 id=\"setting-up-uvu\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#setting-up-uvu\">#</a> Setting up uvu</h3><p>We also maintain a starter template for <code>uvu</code>.</p><p>It includes <a href=\"https://www.npmjs.com/package/solid-dom-testing\">solid-dom-testing</a> to help you write assertions useful with Testing Library.</p><h4 id=\"uvu-with-typescript-(ts-uvu)\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#uvu-with-typescript-(ts-uvu)\">#</a> Uvu with TypeScript (<code>ts-uvu</code>)</h4><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#24292e\">$ npx degit solidjs/templates/ts-uvu my-solid-project</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">$ </span><span style=\"color:#005cc5\">cd</span><span style=\"color:#24292e\"> my-solid-project</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">$ npm install </span><span style=\"color:#6a737d\"># or pnpm install or yarn install</span></span>\n<span class=\"line\"></span></code></pre><h4 id=\"uvu-coverage-reports\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#uvu-coverage-reports\">#</a> Uvu coverage Reports</h4><blockquote> <p>Unfortunately, due to a <a href=\"https://github.com/babel/babel/issues/4289\">limitation of babel</a>, we cannot get source maps output for transpiled JSX, which will result in components to show zero coverage. It will work for non-JSX code, though.</p> </blockquote><p>If you want to check code coverage of your tests, the favorite tool for uvu is c8. To install and set it up, run:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">&gt;</span><span style=\"color:#24292e\"> npm i --save-dev c8 </span><span style=\"color:#6a737d\"># or yarn add -D or pnpm</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">&gt;</span><span style=\"color:#24292e\"> npm set-script </span><span style=\"color:#032f62\">&quot;test:coverage&quot;</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;c8 uvu -r solid-register&quot;</span></span>\n<span class=\"line\"></span></code></pre><p>Now if you <code>npm run test:coverage</code>, you'll see the test coverage.</p><p>If you want nice HTML coverage reports, you can use <code>c8 -r html</code> instead of <code>c8</code> to enable the html reporter.</p><h4 id=\"watch-mode\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#watch-mode\">#</a> Watch Mode</h4><p><code>uvu</code> does not have a watch mode out of the box, but you can use <code>chokidar-cli</code> to do the same:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">&gt;</span><span style=\"color:#24292e\"> npm i --save-dev chokidar-cli </span><span style=\"color:#6a737d\"># or yarn add -D or pnpm</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">&gt;</span><span style=\"color:#24292e\"> npm set-script </span><span style=\"color:#032f62\">&quot;test:watch&quot;</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;chokidar src/**/*.ts src/**/*.tsx -c </span><span style=\"color:#005cc5\">\\&quot;</span><span style=\"color:#032f62\">uvu -r solid-register</span><span style=\"color:#005cc5\">\\&quot;</span></span>\n<span class=\"line\"><span style=\"color:#032f62\"># use .js/.jsx instead of .ts/.tsx</span></span>\n<span class=\"line\"></span></code></pre><p>Now if you run <code>npm run test:watch</code>, the tests will run every time you change a file.</p><h2 id=\"testing-patterns-and-best-practices\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#testing-patterns-and-best-practices\">#</a> Testing Patterns and Best Practices</h2><p>Now that you have installed your testing tools, you should start to use them. In order to make this easier, Solid supports a few nice patterns.</p><h3 id=\"testing-reactive-state\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#testing-reactive-state\">#</a> Testing Reactive State</h3><p>You may want to keep parts of your state separate from the components for ease of maintenance or being able to support multiple views. In this case, the interface against which you are testing is the state itself. Keep in mind that out of a <a href=\"https://www.solidjs.com/docs/latest/api#createroot\">reactive root</a> your state is not tracked and updates won't trigger effects and memos.</p><p>Also, since effects trigger asynchronously, it can help to wrap our assertions in a final effect. Alternatively, to observe a sequence of effects over multiple changes, it can help to return the necessary tools from <code>createRoot</code> and execute them in an async test function (as <code>createRoot</code> itself cannot take an <code>async</code> function).</p><p>As an example, let's test <code>createLocalStorage</code> from the <a href=\"https://www.solidjs.com/examples/todos\">todo example</a>:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { createEffect } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;solid-js&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { createStore, Store, SetStoreFunction } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;solid-js/store&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#d73a49\">export</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">function</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createLocalStore</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#e36209\">T</span><span style=\"color:#24292e\">&gt;(</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#e36209\">initState</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">T</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">)</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> [</span><span style=\"color:#e36209\">Store</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#e36209\">T</span><span style=\"color:#24292e\">&gt;, </span><span style=\"color:#e36209\">SetStoreFunction</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#e36209\">T</span><span style=\"color:#24292e\">&gt;] {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> [</span><span style=\"color:#005cc5\">state</span><span style=\"color:#24292e\">, </span><span style=\"color:#005cc5\">setState</span><span style=\"color:#24292e\">] </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createStore</span><span style=\"color:#24292e\">(initState);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">if</span><span style=\"color:#24292e\"> (localStorage.todos) </span><span style=\"color:#6f42c1\">setState</span><span style=\"color:#24292e\">(</span><span style=\"color:#005cc5\">JSON</span><span style=\"color:#24292e\">.</span><span style=\"color:#005cc5\">parse</span><span style=\"color:#24292e\">(localStorage.todos));</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">createEffect</span><span style=\"color:#24292e\">(() </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> (localStorage.todos </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">JSON</span><span style=\"color:#24292e\">.</span><span style=\"color:#005cc5\">stringify</span><span style=\"color:#24292e\">(state)));</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">return</span><span style=\"color:#24292e\"> [state, setState];</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">}</span></span>\n<span class=\"line\"></span></code></pre><p>Instead of creating a TODO component, we can test this model in isolation; when we do that, we need to keep in mind that 1. reactive changes only work when they have a tracking context provided by <code>render</code> or <code>createRoot</code> and 2. are asynchronous, but we can use <code>createEffect</code> to catch them. Using <code>createRoot</code> has the advantage that we can trigger the disposal manually:</p><h4 id=\"testing-with-jest\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#testing-with-jest\">#</a> Testing with Jest</h4><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { createLocalStore } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;./main.tsx&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { createRoot, createEffect } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;solid-js&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#6f42c1\">describe</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;createLocalStore&quot;</span><span style=\"color:#24292e\">, () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">beforeEach</span><span style=\"color:#24292e\">(() </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    localStorage.</span><span style=\"color:#6f42c1\">removeItem</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;todos&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  });</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">initialState</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    todos: [],</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    newTitle: </span><span style=\"color:#032f62\">&quot;&quot;</span><span style=\"color:#24292e\">,</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  };</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">test</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;it reads pre-existing state from localStorage&quot;</span><span style=\"color:#24292e\">, () </span><span style=\"color:#d73a49\">=&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">createRoot</span><span style=\"color:#24292e\">((</span><span style=\"color:#e36209\">dispose</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">savedState</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> { todos: [], newTitle: </span><span style=\"color:#032f62\">&quot;saved&quot;</span><span style=\"color:#24292e\"> };</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      localStorage.</span><span style=\"color:#6f42c1\">setItem</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;todos&quot;</span><span style=\"color:#24292e\">, </span><span style=\"color:#005cc5\">JSON</span><span style=\"color:#24292e\">.</span><span style=\"color:#005cc5\">stringify</span><span style=\"color:#24292e\">(savedState));</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> [</span><span style=\"color:#005cc5\">state</span><span style=\"color:#24292e\">] </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createLocalStore</span><span style=\"color:#24292e\">(initialState);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#6f42c1\">expect</span><span style=\"color:#24292e\">(state).</span><span style=\"color:#6f42c1\">toEqual</span><span style=\"color:#24292e\">(savedState);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#6f42c1\">dispose</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    }));</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">test</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;it stores new state to localStorage&quot;</span><span style=\"color:#24292e\">, () </span><span style=\"color:#d73a49\">=&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">createRoot</span><span style=\"color:#24292e\">((</span><span style=\"color:#e36209\">dispose</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> [</span><span style=\"color:#005cc5\">state</span><span style=\"color:#24292e\">, </span><span style=\"color:#005cc5\">setState</span><span style=\"color:#24292e\">] </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createLocalStore</span><span style=\"color:#24292e\">(initialState);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#6f42c1\">setState</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;newTitle&quot;</span><span style=\"color:#24292e\">, </span><span style=\"color:#032f62\">&quot;updated&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#6a737d\">// to catch an effect, use an effect</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#d73a49\">return</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">new</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">Promise</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#005cc5\">void</span><span style=\"color:#24292e\">&gt;((</span><span style=\"color:#e36209\">resolve</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        </span><span style=\"color:#6f42c1\">createEffect</span><span style=\"color:#24292e\">(() </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          </span><span style=\"color:#6f42c1\">expect</span><span style=\"color:#24292e\">(</span><span style=\"color:#005cc5\">JSON</span><span style=\"color:#24292e\">.</span><span style=\"color:#005cc5\">parse</span><span style=\"color:#24292e\">(localStorage.todos </span><span style=\"color:#d73a49\">||</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;&quot;</span><span style=\"color:#24292e\">)).</span><span style=\"color:#6f42c1\">toEqual</span><span style=\"color:#24292e\">({</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">            todos: [],</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">            newTitle: </span><span style=\"color:#032f62\">&quot;updated&quot;</span><span style=\"color:#24292e\">,</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          });</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          </span><span style=\"color:#6f42c1\">dispose</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          </span><span style=\"color:#6f42c1\">resolve</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        })</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      );</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    }));</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">test</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;it updates state multiple times&quot;</span><span style=\"color:#24292e\">, </span><span style=\"color:#d73a49\">async</span><span style=\"color:#24292e\"> () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> { </span><span style=\"color:#005cc5\">dispose</span><span style=\"color:#24292e\">, </span><span style=\"color:#005cc5\">setState</span><span style=\"color:#24292e\"> } </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createRoot</span><span style=\"color:#24292e\">((</span><span style=\"color:#e36209\">dispose</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> [</span><span style=\"color:#005cc5\">state</span><span style=\"color:#24292e\">, </span><span style=\"color:#005cc5\">setState</span><span style=\"color:#24292e\">] </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createLocalStore</span><span style=\"color:#24292e\">(initialState);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#d73a49\">return</span><span style=\"color:#24292e\"> { dispose, setState };</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    });</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">setState</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;newTitle&quot;</span><span style=\"color:#24292e\">, </span><span style=\"color:#032f62\">&quot;first&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6a737d\">// wait a tick to resolve all effects</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">await</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">new</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">Promise</span><span style=\"color:#24292e\">((</span><span style=\"color:#e36209\">done</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">setTimeout</span><span style=\"color:#24292e\">(done, </span><span style=\"color:#005cc5\">0</span><span style=\"color:#24292e\">));</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">expect</span><span style=\"color:#24292e\">(</span><span style=\"color:#005cc5\">JSON</span><span style=\"color:#24292e\">.</span><span style=\"color:#005cc5\">parse</span><span style=\"color:#24292e\">(localStorage.todos </span><span style=\"color:#d73a49\">||</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;&quot;</span><span style=\"color:#24292e\">)).</span><span style=\"color:#6f42c1\">toEqual</span><span style=\"color:#24292e\">({</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      todos: [],</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      newTitle: </span><span style=\"color:#032f62\">&quot;first&quot;</span><span style=\"color:#24292e\">,</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    });</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">setState</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;newTitle&quot;</span><span style=\"color:#24292e\">, </span><span style=\"color:#032f62\">&quot;second&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">await</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">new</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">Promise</span><span style=\"color:#24292e\">((</span><span style=\"color:#e36209\">done</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">setTimeout</span><span style=\"color:#24292e\">(done, </span><span style=\"color:#005cc5\">0</span><span style=\"color:#24292e\">));</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">expect</span><span style=\"color:#24292e\">(</span><span style=\"color:#005cc5\">JSON</span><span style=\"color:#24292e\">.</span><span style=\"color:#005cc5\">parse</span><span style=\"color:#24292e\">(localStorage.todos </span><span style=\"color:#d73a49\">||</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;&quot;</span><span style=\"color:#24292e\">)).</span><span style=\"color:#6f42c1\">toEqual</span><span style=\"color:#24292e\">({</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      todos: [],</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      newTitle: </span><span style=\"color:#032f62\">&quot;first&quot;</span><span style=\"color:#24292e\">,</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    });</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">dispose</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  });</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">});</span></span>\n<span class=\"line\"></span></code></pre><h4 id=\"testing-with-uvu\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#testing-with-uvu\">#</a> Testing with uvu</h4><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { createLocalStore } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;./main&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { suite } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;uvu&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">*</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">as</span><span style=\"color:#24292e\"> assert </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;uvu/assert&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { createEffect, createRoot } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;solid-js&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">todoTest</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">suite</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;createLocalStore&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">todoTest.before.</span><span style=\"color:#6f42c1\">each</span><span style=\"color:#24292e\">(() </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  localStorage.</span><span style=\"color:#6f42c1\">removeItem</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;todos&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">});</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">initialState</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  todos: [],</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  newTitle: </span><span style=\"color:#032f62\">&quot;&quot;</span><span style=\"color:#24292e\">,</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">};</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#6f42c1\">todoTest</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;it reads pre-existing state from localStorage&quot;</span><span style=\"color:#24292e\">, () </span><span style=\"color:#d73a49\">=&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">createRoot</span><span style=\"color:#24292e\">((</span><span style=\"color:#e36209\">dispose</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">savedState</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> { todos: [], newTitle: </span><span style=\"color:#032f62\">&quot;saved&quot;</span><span style=\"color:#24292e\"> };</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    localStorage.</span><span style=\"color:#6f42c1\">setItem</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;todos&quot;</span><span style=\"color:#24292e\">, </span><span style=\"color:#005cc5\">JSON</span><span style=\"color:#24292e\">.</span><span style=\"color:#005cc5\">stringify</span><span style=\"color:#24292e\">(savedState));</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> [</span><span style=\"color:#005cc5\">state</span><span style=\"color:#24292e\">] </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createLocalStore</span><span style=\"color:#24292e\">(initialState);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    assert.</span><span style=\"color:#6f42c1\">equal</span><span style=\"color:#24292e\">(state, savedState);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">dispose</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  })</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#6f42c1\">todoTest</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;it stores new state to localStorage&quot;</span><span style=\"color:#24292e\">, () </span><span style=\"color:#d73a49\">=&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">createRoot</span><span style=\"color:#24292e\">((</span><span style=\"color:#e36209\">dispose</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> [</span><span style=\"color:#005cc5\">_</span><span style=\"color:#24292e\">, </span><span style=\"color:#005cc5\">setState</span><span style=\"color:#24292e\">] </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createLocalStore</span><span style=\"color:#24292e\">(initialState);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">setState</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;newTitle&quot;</span><span style=\"color:#24292e\">, </span><span style=\"color:#032f62\">&quot;updated&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6a737d\">// to catch an effect, we need an effect</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">return</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">new</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">Promise</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#005cc5\">void</span><span style=\"color:#24292e\">&gt;((</span><span style=\"color:#e36209\">resolve</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#6f42c1\">createEffect</span><span style=\"color:#24292e\">(() </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        assert.</span><span style=\"color:#6f42c1\">equal</span><span style=\"color:#24292e\">(</span><span style=\"color:#005cc5\">JSON</span><span style=\"color:#24292e\">.</span><span style=\"color:#005cc5\">parse</span><span style=\"color:#24292e\">(localStorage.todos </span><span style=\"color:#d73a49\">||</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;&quot;</span><span style=\"color:#24292e\">), {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          todos: [],</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          newTitle: </span><span style=\"color:#032f62\">&quot;updated&quot;</span><span style=\"color:#24292e\">,</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        });</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        </span><span style=\"color:#6f42c1\">dispose</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        </span><span style=\"color:#6f42c1\">resolve</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      })</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    );</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  })</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">todoTest.</span><span style=\"color:#6f42c1\">run</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"></span></code></pre><h3 id=\"testing-directives\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#testing-directives\">#</a> Testing directives</h3><p><a href=\"https://www.solidjs.com/docs/latest/api#use%3A___\">Directives</a> allow using refs in a reusable way. They are basically functions that follow the pattern <code>(ref: HTMLElement, data: Accessor&lt;any&gt;) =&gt; void</code>. In our <a href=\"https://www.solidjs.com/tutorial/bindings_directives?solved\">directives tutorial</a>, we define the <code>clickOutside</code> directive that should call the callback wrapped in the accessor argument.</p><p>We could now create a component and use the directive in there, but then we'd be testing the use of directives instead of directly testing the directive. It's simpler to test the surface of the directive by providing a mounted node and the accessor:</p><h4 id=\"testing-with-jest-1\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#testing-with-jest-1\">#</a> Testing with Jest</h4><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#6a737d\">// click-outside.test.ts</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> clickOutside </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;click-outside&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { createRoot } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;solid-js&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { fireEvent } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;solid-testing-library&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#6f42c1\">describe</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;clickOutside&quot;</span><span style=\"color:#24292e\">, () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">ref</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> document.</span><span style=\"color:#6f42c1\">createElement</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;div&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">beforeAll</span><span style=\"color:#24292e\">(() </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    document.body.</span><span style=\"color:#6f42c1\">appendChild</span><span style=\"color:#24292e\">(ref);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  });</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">afterAll</span><span style=\"color:#24292e\">(() </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    document.body.</span><span style=\"color:#6f42c1\">removeChild</span><span style=\"color:#24292e\">(ref);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  });</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">test</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;will trigger on click outside&quot;</span><span style=\"color:#24292e\">, () </span><span style=\"color:#d73a49\">=&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">createRoot</span><span style=\"color:#24292e\">(</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      (</span><span style=\"color:#e36209\">dispose</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        </span><span style=\"color:#d73a49\">new</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">Promise</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#005cc5\">void</span><span style=\"color:#24292e\">&gt;((</span><span style=\"color:#e36209\">resolve</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          </span><span style=\"color:#d73a49\">let</span><span style=\"color:#24292e\"> clickedOutside </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">false</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          </span><span style=\"color:#6f42c1\">clickOutside</span><span style=\"color:#24292e\">(ref, () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">            clickedOutside </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">true</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          });</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          document.body.</span><span style=\"color:#6f42c1\">addEventListener</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;click&quot;</span><span style=\"color:#24292e\">, () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">            </span><span style=\"color:#6f42c1\">expect</span><span style=\"color:#24292e\">(clickedOutside).</span><span style=\"color:#6f42c1\">toBeTruthy</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">            </span><span style=\"color:#6f42c1\">dispose</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">            </span><span style=\"color:#6f42c1\">resolve</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          });</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          fireEvent.</span><span style=\"color:#6f42c1\">click</span><span style=\"color:#24292e\">(document.body);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        })</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    ));</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">test</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;will not trigger on click inside&quot;</span><span style=\"color:#24292e\">, () </span><span style=\"color:#d73a49\">=&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">createRoot</span><span style=\"color:#24292e\">(</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      (</span><span style=\"color:#e36209\">dispose</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        </span><span style=\"color:#d73a49\">new</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">Promise</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#005cc5\">void</span><span style=\"color:#24292e\">&gt;((</span><span style=\"color:#e36209\">resolve</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          </span><span style=\"color:#d73a49\">let</span><span style=\"color:#24292e\"> clickedOutside </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">false</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          </span><span style=\"color:#6f42c1\">clickOutside</span><span style=\"color:#24292e\">(ref, () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">            clickedOutside </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">true</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          });</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          ref.</span><span style=\"color:#6f42c1\">addEventListener</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;click&quot;</span><span style=\"color:#24292e\">, () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">            </span><span style=\"color:#6f42c1\">expect</span><span style=\"color:#24292e\">(clickedOutside).</span><span style=\"color:#6f42c1\">toBeFalsy</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">            </span><span style=\"color:#6f42c1\">dispose</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">            </span><span style=\"color:#6f42c1\">resolve</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          });</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          fireEvent.</span><span style=\"color:#6f42c1\">click</span><span style=\"color:#24292e\">(ref);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        })</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    ));</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">});</span></span>\n<span class=\"line\"></span></code></pre><h4 id=\"testing-with-uvu-1\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#testing-with-uvu-1\">#</a> Testing with uvu</h4><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#6a737d\">// click-outside.test.ts</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> clickOutside </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;click-outside.tsx&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { createRoot } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;solid-js&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { fireEvent } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;solid-testing-library&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">clickTest</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">suite</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;clickOutside&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">ref</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> document.</span><span style=\"color:#6f42c1\">createElement</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;div&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">clickTest.</span><span style=\"color:#6f42c1\">before</span><span style=\"color:#24292e\">(() </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  document.body.</span><span style=\"color:#6f42c1\">appendChild</span><span style=\"color:#24292e\">(ref);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">});</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">clickTest.</span><span style=\"color:#6f42c1\">after</span><span style=\"color:#24292e\">(() </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  document.body.</span><span style=\"color:#6f42c1\">removeChild</span><span style=\"color:#24292e\">(ref);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">});</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#6f42c1\">clickTest</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;will trigger on click outside&quot;</span><span style=\"color:#24292e\">, () </span><span style=\"color:#d73a49\">=&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">createRoot</span><span style=\"color:#24292e\">(</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    (</span><span style=\"color:#e36209\">dispose</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#d73a49\">new</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">Promise</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#005cc5\">void</span><span style=\"color:#24292e\">&gt;((</span><span style=\"color:#e36209\">resolve</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        </span><span style=\"color:#d73a49\">let</span><span style=\"color:#24292e\"> clickedOutside </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">false</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        </span><span style=\"color:#6f42c1\">clickOutside</span><span style=\"color:#24292e\">(ref, () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          clickedOutside </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">true</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        });</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        document.body.</span><span style=\"color:#6f42c1\">addEventListener</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;click&quot;</span><span style=\"color:#24292e\">, () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          assert.</span><span style=\"color:#6f42c1\">ok</span><span style=\"color:#24292e\">(clickedOutside);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          </span><span style=\"color:#6f42c1\">dispose</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          </span><span style=\"color:#6f42c1\">resolve</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        });</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        fireEvent.</span><span style=\"color:#6f42c1\">click</span><span style=\"color:#24292e\">(document.body);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      })</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  )</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#6f42c1\">clickTest</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;will not trigger on click inside&quot;</span><span style=\"color:#24292e\">, () </span><span style=\"color:#d73a49\">=&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">createRoot</span><span style=\"color:#24292e\">(</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    (</span><span style=\"color:#e36209\">dispose</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#d73a49\">new</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">Promise</span><span style=\"color:#24292e\">&lt;</span><span style=\"color:#005cc5\">void</span><span style=\"color:#24292e\">&gt;((</span><span style=\"color:#e36209\">resolve</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        </span><span style=\"color:#d73a49\">let</span><span style=\"color:#24292e\"> clickedOutside </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">false</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        </span><span style=\"color:#6f42c1\">clickOutside</span><span style=\"color:#24292e\">(ref, () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          clickedOutside </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">true</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        });</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        ref.</span><span style=\"color:#6f42c1\">addEventListener</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;click&quot;</span><span style=\"color:#24292e\">, () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          assert.</span><span style=\"color:#6f42c1\">is</span><span style=\"color:#24292e\">(clickedOutside, </span><span style=\"color:#005cc5\">false</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          </span><span style=\"color:#6f42c1\">dispose</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">          </span><span style=\"color:#6f42c1\">resolve</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        });</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">        fireEvent.</span><span style=\"color:#6f42c1\">click</span><span style=\"color:#24292e\">(ref);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      })</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  )</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">clickTest.</span><span style=\"color:#6f42c1\">run</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"></span></code></pre><h3 id=\"testing-components\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#testing-components\">#</a> Testing components</h3><p>Let's take a simple click-counter component that we want to test:</p><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#6a737d\">// main.tsx</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { createSignal, Component } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;solid-js&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#d73a49\">export</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">Counter</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">Component</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> [</span><span style=\"color:#005cc5\">count</span><span style=\"color:#24292e\">, </span><span style=\"color:#005cc5\">setCount</span><span style=\"color:#24292e\">] </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">createSignal</span><span style=\"color:#24292e\">(</span><span style=\"color:#005cc5\">0</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">return</span><span style=\"color:#24292e\"> (</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">&lt;</span><span style=\"color:#24292e\">div role</span><span style=\"color:#d73a49\">=</span><span style=\"color:#032f62\">&quot;button&quot;</span><span style=\"color:#24292e\"> onClick</span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\">{() =&gt; </span><span style=\"color:#6f42c1\">setCount</span><span style=\"color:#24292e\">((</span><span style=\"color:#e36209\">c</span><span style=\"color:#24292e\">) </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> </span><span style=\"color:#e36209\">c</span><span style=\"color:#24292e\"> + </span><span style=\"color:#005cc5\">1</span><span style=\"color:#24292e\">)}</span><span style=\"color:#d73a49\">&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">      </span><span style=\"color:#e36209\">Count</span><span style=\"color:#d73a49\">:</span><span style=\"color:#24292e\"> {</span><span style=\"color:#6f42c1\">count</span><span style=\"color:#24292e\">()}</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">&lt;/</span><span style=\"color:#24292e\">div</span><span style=\"color:#d73a49\">&gt;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  );</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">};</span></span>\n<span class=\"line\"></span></code></pre><p>Here we use <code>solid-testing-library</code>. It's most important helpers are <code>render</code> to render a component to the DOM in a managed way, <code>fireEvent</code> to dispatch events in a way that resembles actual user events and <code>screen</code> to provide global selectors. We also use helpful assertions added to <code>expect</code> provided by <code>@testing-library/jest-dom</code>.</p><h4 id=\"testing-with-jest-2\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#testing-with-jest-2\">#</a> Testing with Jest</h4><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#6a737d\">// main.test.tsx</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { Counter } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;./main&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { cleanup, fireEvent, render, screen } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;solid-testing-library&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#6f42c1\">describe</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;Counter&quot;</span><span style=\"color:#24292e\">, () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">afterEach</span><span style=\"color:#24292e\">(cleanup);</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">test</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;it starts with zero&quot;</span><span style=\"color:#24292e\">, () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">render</span><span style=\"color:#24292e\">(() </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> &lt;</span><span style=\"color:#e36209\">Counter</span><span style=\"color:#24292e\"> /&gt;);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">button</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> screen.</span><span style=\"color:#6f42c1\">getByRole</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;button&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">expect</span><span style=\"color:#24292e\">(button).</span><span style=\"color:#6f42c1\">toBeInTheDocument</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">expect</span><span style=\"color:#24292e\">(button).</span><span style=\"color:#6f42c1\">toHaveTextContent</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;Count: 0&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  });</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">test</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;it increases its value on click&quot;</span><span style=\"color:#24292e\">, </span><span style=\"color:#d73a49\">async</span><span style=\"color:#24292e\"> () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">render</span><span style=\"color:#24292e\">(() </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> &lt;</span><span style=\"color:#e36209\">Counter</span><span style=\"color:#24292e\"> /&gt;);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">button</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> screen.</span><span style=\"color:#6f42c1\">getByRole</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;button&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    fireEvent.</span><span style=\"color:#6f42c1\">click</span><span style=\"color:#24292e\">(button);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6a737d\">// the event loop takes one Promise to resolve to be finished</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">await</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">Promise</span><span style=\"color:#24292e\">.</span><span style=\"color:#6f42c1\">resolve</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">expect</span><span style=\"color:#24292e\">(button).</span><span style=\"color:#6f42c1\">toHaveTextContent</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;Count: 1&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    fireEvent.</span><span style=\"color:#6f42c1\">click</span><span style=\"color:#24292e\">(button);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#d73a49\">await</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">Promise</span><span style=\"color:#24292e\">.</span><span style=\"color:#6f42c1\">resolve</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">expect</span><span style=\"color:#24292e\">(button).</span><span style=\"color:#6f42c1\">toHaveTextContent</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;Count: 2&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  });</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">});</span></span>\n<span class=\"line\"></span></code></pre><h4 id=\"testing-with-uvu-2\" tabindex=\"-1\"><a class=\"header-anchor\" href=\"#testing-with-uvu-2\">#</a> Testing with uvu</h4><pre class=\"shiki\" style=\"background-color:transparent\"><code><span class=\"line\"><span style=\"color:#6a737d\">// main.test.tsx</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { suite } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;uvu&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">*</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">as</span><span style=\"color:#24292e\"> assert </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;uvu/assert&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { Counter } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;main&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { fireEvent, render, screen } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;solid-testing-library&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"><span style=\"color:#d73a49\">import</span><span style=\"color:#24292e\"> { isInDocument, hasTextContent } </span><span style=\"color:#d73a49\">from</span><span style=\"color:#24292e\"> </span><span style=\"color:#032f62\">&quot;solid-dom-testing&quot;</span><span style=\"color:#24292e\">;</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">testCounter</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">suite</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;Counter&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">testCounter.after.</span><span style=\"color:#6f42c1\">each</span><span style=\"color:#24292e\">(cleanup);</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#6f42c1\">testCounter</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;it starts with zero&quot;</span><span style=\"color:#24292e\">, () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> { </span><span style=\"color:#005cc5\">getByRole</span><span style=\"color:#24292e\"> } </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">render</span><span style=\"color:#24292e\">(() </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> &lt;</span><span style=\"color:#e36209\">Counter</span><span style=\"color:#24292e\"> /&gt;);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">button</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> </span><span style=\"color:#6f42c1\">getByRole</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;button&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  assert.</span><span style=\"color:#6f42c1\">ok</span><span style=\"color:#24292e\">(</span><span style=\"color:#6f42c1\">isInDocument</span><span style=\"color:#24292e\">(button), </span><span style=\"color:#032f62\">&quot;button not in dom&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  assert.</span><span style=\"color:#6f42c1\">ok</span><span style=\"color:#24292e\">(</span><span style=\"color:#6f42c1\">hasTextContent</span><span style=\"color:#24292e\">(button, </span><span style=\"color:#032f62\">&quot;Count: 0&quot;</span><span style=\"color:#24292e\">), </span><span style=\"color:#032f62\">&quot;wrong text content&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">});</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#6f42c1\">testCounter</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;it increases its value on click&quot;</span><span style=\"color:#24292e\">, </span><span style=\"color:#d73a49\">async</span><span style=\"color:#24292e\"> () </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> {</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6f42c1\">render</span><span style=\"color:#24292e\">(() </span><span style=\"color:#d73a49\">=&gt;</span><span style=\"color:#24292e\"> &lt;</span><span style=\"color:#e36209\">Counter</span><span style=\"color:#24292e\"> /&gt;);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">const</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">button</span><span style=\"color:#24292e\"> </span><span style=\"color:#d73a49\">=</span><span style=\"color:#24292e\"> screen.</span><span style=\"color:#6f42c1\">getByRole</span><span style=\"color:#24292e\">(</span><span style=\"color:#032f62\">&quot;button&quot;</span><span style=\"color:#24292e\">);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  fireEvent.</span><span style=\"color:#6f42c1\">click</span><span style=\"color:#24292e\">(button);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#6a737d\">// the event loop takes one Promise to resolve to be finished</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">await</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">Promise</span><span style=\"color:#24292e\">.</span><span style=\"color:#6f42c1\">resolve</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  assert.</span><span style=\"color:#6f42c1\">ok</span><span style=\"color:#24292e\">(</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">hasTextContent</span><span style=\"color:#24292e\">(button, </span><span style=\"color:#032f62\">&quot;Count: 1&quot;</span><span style=\"color:#24292e\">),</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#032f62\">&quot;not count 1 after first click&quot;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  );</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  fireEvent.</span><span style=\"color:#6f42c1\">click</span><span style=\"color:#24292e\">(button);</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  </span><span style=\"color:#d73a49\">await</span><span style=\"color:#24292e\"> </span><span style=\"color:#005cc5\">Promise</span><span style=\"color:#24292e\">.</span><span style=\"color:#6f42c1\">resolve</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  assert.</span><span style=\"color:#6f42c1\">ok</span><span style=\"color:#24292e\">(</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#6f42c1\">hasTextContent</span><span style=\"color:#24292e\">(button, </span><span style=\"color:#032f62\">&quot;Count: 2&quot;</span><span style=\"color:#24292e\">),</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">    </span><span style=\"color:#032f62\">&quot;not count 2 after first click&quot;</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">  );</span></span>\n<span class=\"line\"><span style=\"color:#24292e\">});</span></span>\n<span class=\"line\"></span>\n<span class=\"line\"><span style=\"color:#24292e\">testCounter.</span><span style=\"color:#6f42c1\">run</span><span style=\"color:#24292e\">();</span></span>\n<span class=\"line\"></span></code></pre></section>";
var attributes = {
	title: "Testing Solid",
	description: "How to test your Solid app",
	sort: 4
};
var sections = [
	{
		slug: "testing-solid",
		title: "Testing Solid",
		level: 1,
		children: [
			{
				slug: "testing-setup",
				title: "Testing Setup",
				level: 2,
				children: [
					{
						slug: "setting-up-jest",
						title: "Setting up Jest",
						level: 3,
						children: [
							{
								slug: "jest-with-typescript-(ts-jest)",
								title: "Jest with TypeScript (ts-jest)",
								level: 4,
								children: [
								]
							}
						]
					},
					{
						slug: "setting-up-uvu",
						title: "Setting up uvu",
						level: 3,
						children: [
							{
								slug: "uvu-with-typescript-(ts-uvu)",
								title: "Uvu with TypeScript (ts-uvu)",
								level: 4,
								children: [
								]
							},
							{
								slug: "uvu-coverage-reports",
								title: "Uvu coverage Reports",
								level: 4,
								children: [
								]
							},
							{
								slug: "watch-mode",
								title: "Watch Mode",
								level: 4,
								children: [
								]
							}
						]
					}
				]
			},
			{
				slug: "testing-patterns-and-best-practices",
				title: "Testing Patterns and Best Practices",
				level: 2,
				children: [
					{
						slug: "testing-reactive-state",
						title: "Testing Reactive State",
						level: 3,
						children: [
							{
								slug: "testing-with-jest",
								title: "Testing with Jest",
								level: 4,
								children: [
								]
							},
							{
								slug: "testing-with-uvu",
								title: "Testing with uvu",
								level: 4,
								children: [
								]
							}
						]
					},
					{
						slug: "testing-directives",
						title: "Testing directives",
						level: 3,
						children: [
							{
								slug: "testing-with-jest-1",
								title: "Testing with Jest",
								level: 4,
								children: [
								]
							},
							{
								slug: "testing-with-uvu-1",
								title: "Testing with uvu",
								level: 4,
								children: [
								]
							}
						]
					},
					{
						slug: "testing-components",
						title: "Testing components",
						level: 3,
						children: [
							{
								slug: "testing-with-jest-2",
								title: "Testing with Jest",
								level: 4,
								children: [
								]
							},
							{
								slug: "testing-with-uvu-2",
								title: "Testing with uvu",
								level: 4,
								children: [
								]
							}
						]
					}
				]
			}
		]
	}
];
var testing = {
	html: html,
	attributes: attributes,
	sections: sections
};

export { attributes, testing as default, html, sections };
